/* (C) 2018, Andreas Huber, UniBw München LRT 1, Germany. All rights reserved. */

/*
 * Commercial license:
 *
 * To get more information about the commercial licence and the commercial use of OCPbasic and IPbasic please contact
 *      info@ocpbasic.com
 *
 * Open source license: (LGPLv3)
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 3.0 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file DiscreteDefs.h
 * Header Definitions of DiscreteDefs.
 *
 * @author A. Huber
 * @since 27.09.2018
 */

#ifndef DISKRDEFINITIONEN_H
#define DISKRDEFINITIONEN_H

namespace OCPbasic
{

    class DiscreteDefs
    {
      public:
        /**
         * \brief The BoundType enum:
         *        Defines the type BoundType to identify the constraints.
         */
        enum BoundType
        {
            /// Inequality has upper and lower bound.
            INEQUALITY,
            /// Inequality has the same upper and lower bound => equality.
            EQUATION,
            /// Inequality has only lower bound.
            LOWER_BARRIER,
            /// Inequality has only upper bound.
            UPPER_BARRIER,
            /// Inequality is unbounded.
            UNCONSTRAINED
        };

        /**
         * \brief Definition of the BoundDimension struct.
         */
        struct BoundDimension
        {
            /**
             * \brief BoundDimension:
             *        Constructor ot the struct BoundDimension.
             *
             * \param[in] N             Number of variables.
             * \param[in] N_Dim_ROW     Resulting number of rows.
             * \param[in] N_UGL         Number of inequalitys.
             * \param[in] N_GL          Number of equalitys.
             * \param[in] N_US          Number of pure lower bounds.
             * \param[in] N_OS          Number of pure upper bounds.
             * \param[in] N_ZL          Number of unbounded inequalitys.
             *
             */
            BoundDimension(int_type N = 0, int_type N_Dim_ROW = 0, int_type N_UGL = 0, int_type N_GL = 0, int_type N_US = 0, int_type N_OS = 0, int_type N_ZL = 0)
                : N(N)
                , N_Dim_ROW(N_Dim_ROW)
                , N_UGL(N_UGL)
                , N_GL(N_GL)
                , N_US(N_US)
                , N_OS(N_OS)
                , N_ZL(N_ZL)
            {
            }

            /// Number of variables.
            int_type N;
            /// Resulting number of rows.
            int_type N_Dim_ROW;
            /// Number of inequalitys.
            int_type N_UGL;
            /// Number of equalitys.
            int_type N_GL;
            /// Number of pure lower bounds.
            int_type N_US;
            /// Number of pure upper bounds.
            int_type N_OS;
            /// Number of unbounded inequalitys.
            int_type N_ZL;
        };
    };

} // namespace OCPbasic

#endif // DISKRDEFINITIONEN_H
