/* (C) 2019, Andreas Huber, UniBw München LRT 1, Germany. All rights reserved. */

/*
 * Commercial license:
 *
 * To get more information about the commercial licence and the commercial use of OCPbasic and IPbasic please contact
 *      info@ocpbasic.com
 *
 * Open source license: (LGPLv3)
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 3.0 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file OCP_Params.h
 * Header definition of the class OCP_Params.
 *
 * @author A. Huber
 * @since 13.05.2019
 */

#ifndef OCP_PARAMS_H
#define OCP_PARAMS_H
#include <IPbasic/IPbasic_Parameter.h>

namespace OCPbasic
{

    /**
     * \brief Flags for the definition of regularization flags.
     */
    typedef enum
    {
        /// No regularization.
        REGULARIZE_NONE = 0,
        /// Regularize hessian with partial eigenvalues.
        REGULARIZE_PARTIAL_EIGENS = 1,
        /// Regularize hessian with Gersch Gorin.
        REGULARIZE_GERSCHGORIN = 2,
    } RegularizationFlag;

    /**
     * \brief Flags for the definition of linear solver flags.
     */
    typedef enum
    {
        /// Use hsl ma57 for solving linear systems.
        LINSOL_HSL_MA57 = 0,
        /// Use lapack banded solver to solve kkt like systems.
        LINSOL_LAPACK_BANDED_KKT = 1,
    } LinearSolverFlag;

    template <typename real>
    class OCP_Params : public IPbasic::IPparameter<real>
    {
      public:
        /**
         * \brief OCP_Params:
         *        Constructor of the class OCP_Params.
         */
        OCP_Params()
            : IPbasic::IPparameter<real>()
        {
            this->MyRegularization = REGULARIZE_NONE;
        }

        /**
         * \brief ~OCP_Params:
         *        Virtual destructor of the class OCP_Params.
         */
        virtual ~OCP_Params()
        {
        }

        /// Regularization flag for the hessian.
        RegularizationFlag MyRegularization;
    };

} // namespace OCPbasic

#endif // OCP_PARAMS_H
