/* (C) 2019, Andreas Huber, UniBw München LRT 1, Germany. All rights reserved. */

/*
 * Commercial license:
 *
 * To get more information about the commercial licence and the commercial use of OCPbasic and IPbasic please contact
 *      info@ocpbasic.com
 *
 * Open source license: (LGPLv3)
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 3.0 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library;
 * if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef OCPBASIC_SYMEIGENVALUES_H
#define OCPBASIC_SYMEIGENVALUES_H

#include <IPbasic/IPbasic.h>
#include <lapack_wrapper/lapack_wrapper++.hh>

using IPbasic::int_type;
using lapack_wrapper::Malloc;
using lapack_wrapper::MatrixWrapper;

namespace OCPbasic
{

    template <typename real>
    class SymEigenValues
    {
        Malloc<real> mem_real;
        Malloc<int>  mem_int;
        /// Tolerance in QR.
        real tolQR;
        /// Tolerance for diagonal.
        real tolDiag;
        /// Maximal number of iterations.
        int_type Maxiter;

      public:
        /**
         * \brief SymEigenValues:
         *        Constructor of the class SymEigenValues.
         */
        SymEigenValues();

        /**
         * \brief ~SymEigenValues:
         *        Virtual destructor of the class SymEigenValues.
         */
        virtual ~SymEigenValues();

        /**
         * \brief get_Symmetric_Eigenvalues:
         *        Get the eigenvalues of a symmetric matrix \f$A\f$.
         *
         * \param[in]  A            Symmetric matrix.
         * \param[out] eigenVals    Vector with the eigenvalues
         *
         * @return 0, if the calculation of eigenvalues was successfull.
         *
         */
        int_type get_Symmetric_Eigenvalues(MatrixWrapper<real> const & A, real eigenVals[]);

      private:
        /**
         * \brief get_Diag:
         *        Get diagonal of a matrix \f$A\f$.
         *
         * \param[in]  A    Matrix.
         * \param[out] Diag Diagonal.
         *
         */
        void get_Diag(MatrixWrapper<real> const & A, real Diag[]);

        /**
         * \brief checkMatrix:
         *        Checks wether a matrix has diagonal structure.
         *
         * \param[in] A     Matrix.
         *
         * @return True, if the matrix has a diagonal structure.
         *
         */
        bool checkMatrix(MatrixWrapper<real> const & A);
    };

} // namespace OCPbasic

#endif // OCPBASIC_SYMEIGENVALUES_H
